const express = require('express');
const app = express();
const path = require('path');
const formidable = require('formidable');
const fs = require('fs');
const cheerio = require('cheerio');
const jsStringEscape = require('js-string-escape');
const open = require('open');
const mkdirp = require('mkdirp');
const bodyParser = require('body-parser');

const sceneFilePath = () => path.join(process.cwd(), 'scene.json');

module.exports = {
  usage: {
    commandLine: '',
    description: 'Start image placement tool'
  },
  run: cliArgs => {
    app.use(bodyParser.json());

    const Webpack = require("webpack");
    const webpackMiddleware = require("webpack-dev-middleware");
    const webpackConfig = require("./ui/webpack.config");

    const compiler = Webpack(webpackConfig);
    app.use(webpackMiddleware(compiler));


    app.get('/scene', function(req, res) {
      res.sendFile(path.join(process.cwd(), 'scene.json'));
    });

    app.post('/scene', function(req, res) {
      fs.writeFile(sceneFilePath(), JSON.stringify(req.body, null, 2), err => {
        res.end('success');
      });
    });

    app.use(express.static(path.join(__dirname, 'ui')));

    app.get('/file/*', function(req, res) {
      var filePath = decodeURIComponent(req.path.substring(6));
      res.sendFile(path.join(process.cwd(), 'placed', filePath));
    });

    app.post('/file/:name', function(req, res){

      const placedDirectory = path.join(process.cwd(), 'placed');
      console.log('will create ', placedDirectory);
      mkdirp(placedDirectory, err => {
        if(err) {
          console.error(err);
          res.status(500).send(err);
        }
        else {

        // create an incoming form object
        var form = new formidable.IncomingForm();

        // specify that we want to allow the user to upload multiple files in a single request
        form.multiples = true;

        form.uploadDir = placedDirectory;


        // every time a file has been uploaded successfully,
        // rename it to it's orignal name
        form.on('file', function(field, file) {
          var dest = path.join(form.uploadDir, req.params.name);
          console.log('renaming file to ' + dest);
          fs.rename(file.path, dest);
        });

        // log any errors that occur
        form.on('error', function(err) {
          res.status(500).send(err);
          console.log('An error has occured: \n' + err);
        });

        // once all the files have been uploaded, send a response to the client
        form.on('end', function() {
          res.end('success');
        });

        // parse the incoming request containing the form data
        form.parse(req);
        }
      });
    });


    const port = 3000;
    const url = `http://0.0.0.0:${port}`;
    var server = app.listen(port, function() {
      console.log(`Running image placement GUI at ${url}`);
      open(url);
    });

  }
};

