#!/usr/bin/env node

const fs = require('fs');
const argv = require('yargs').argv;
// const mkdirp = require('mkdirp');
const path = require('path');
const async = require('async');
const sharp = require('sharp');
var sum = require('lodash').sum;
// const _ = require('lodash');


const sceneFilePath = path.join(process.cwd(), 'scene.json');

const Point = require('./Point');

module.exports = {
	usage: {
		commandLine: () => ' --size={rows}x{columns}',
		description: () => 'Create a scene with passed images in a grid'
	},
	run: (cliArgs, {onUsageError}) => {

		require('./readdirVisible')(argv.imagesDir, (err, files) => {

			files.sort();

			if(argv.reverse) {
				files.reverse();
			}

			let scene = [];

			let row = 0;
			let rowHeight;

			let imagePaths = [];
			let imageSizes = [];


			async.eachSeries(files, (filename, cb) => {
				const imagePath = filename;
				sharp(imagePath)
					.metadata()
					.then(metadata => {
						
						if(!rowHeight) {
							rowHeight = metadata.height;
						}

						imagePaths.push(imagePath);
						imageSizes.push([metadata.width, metadata.height]);

						cb();
					})
					.catch(err => {
						console.error('Error while trying to read image ' + imagePath)
					});
			}, err => {
				if(err) {
					console.error(err);
					process.exit(1);
				}
				else {

					const totalWidth = sum(imageSizes.map(size => size[0]));
					const totalUnitWidth = totalWidth / rowHeight;
					const unitWidth = Math.sqrt(totalUnitWidth);
					const pixelWidth = unitWidth * rowHeight;
					console.error({totalWidth, totalUnitWidth, unitWidth, pixelWidth});

					let next = [0, 0];
					const scene = imageSizes.map((size, s) => {
						const placedImage = {
							"imageUrl": imagePaths[s],
							"imageAspect": size[0] / size[1],
							"upperLeft": {
								"x": ''+next[0],
								"y": ''+(next[1]+size[1])
							},
							"lowerRight": {
								"x": ''+(next[0] + size[0]),
								"y": ''+next[1]
							}
						};
						next[0] += size[0];
						if(next[0] > pixelWidth) {
							next[1] -= rowHeight;
							next[0] = 0 - (rowHeight - (next[0] - pixelWidth));
						}
						return placedImage;
					});

					const info = {
						bounds: {
							left: 0,
							right: pixelWidth,
							top: rowHeight,
							bottom: rowHeight - pixelWidth
						},
						aspect: 1
					};

					const sceneJson = JSON.stringify(scene, null, 2);
					fs.writeFile('scene.json', sceneJson, {encoding: 'utf8'}, err => {
						if(err) {
							console.error(err);
							process.exit(1);
						}
						else {
							console.error('Saved scene.json');

							const infoJson = JSON.stringify(info, null, 2);
							fs.writeFile('info.json', infoJson, {encoding: 'utf8'}, err => {
								if(err) {
									console.error(err);
									process.exit(1);
								}
								else {
									console.error('Saved info.json');
								}
							});

						}
					});
				}
			});
		});
	}
}
