#!/usr/bin/env node

const Decimal = require('shardedmapview').DecimalConfigured;
const D = v => new Decimal(v);

const argv = require('yargs').argv;
const mkdirp = require('mkdirp');
const path = require('path');
const async = require('async');
const _ = require('lodash');


const sceneFilePath = path.join(process.cwd(), 'scene.json');

const Point = require('./Point');

module.exports = {
	usage: {
		commandLine: () => '',
		description: () => 'Display information about placed images'
	},
	run: (cliArgs, {onUsageError}) => {

		require('./scene').open(sceneFilePath, (err, scene) => {
			const centroids = scene.images.map(image => Point.average(
				Point(image.upperLeft),
				Point(image.lowerRight)
			));
			console.error('Centroids:', centroids.map(c => c.toString()));
			
			const xs = _.flatten(scene.images.map(image => [
				new Decimal(image.upperLeft.x),
				new Decimal(image.lowerRight.x)
			]));
			const ys = _.flatten(scene.images.map(image => [
				new Decimal(image.upperLeft.y),
				new Decimal(image.lowerRight.y)
			]));
			const bounds = {
				left: Decimal.min.apply(Decimal, xs).toString(),
				right: Decimal.max.apply(Decimal, xs).toString(),
				top: Decimal.max.apply(Decimal, ys).toString(),
				bottom: Decimal.min.apply(Decimal, ys).toString()
			};
			const width = D(bounds.right).minus(bounds.left).toString();
			const height = D(bounds.top).minus(bounds.bottom).toString();
			const aspect = D(width).dividedBy(height).toString();

			console.log(JSON.stringify({
				bounds,
				aspect
			}))
		});

	}
}
