#!/usr/bin/env node

const argv = require('yargs').argv;
const mkdirp = require('mkdirp');
const path = require('path');
const async = require('async');
const _ = require('lodash');

const renderzoomlevel = require('./renderzoomlevel');



const outputDir = path.join(process.cwd(), 'tiles');

const sceneFilePath = path.join(process.cwd(), 'scene.json');

const imageExporters = {
	'png8': require('./canvasToPng8'),
	'jpeg': require('./canvasToJpeg')({
		quality: (+argv.quality) || 50
	})
};

module.exports = {
	usage: {
		commandLine: () => '<format>',
		description: () => 'Render XYZ tiles. <format> must be one of ' + Object.keys(imageExporters).join(', ')
	},
	run: (cliArgs, {onUsageError}) => {

		const imageExporterKey = cliArgs[0];
		const maxZoom = +argv.maxZoom || 70;
		const minZoom = +argv.minZoom || 0;

		console.log(`min zoom is ${minZoom}. Configure with --min-zoom`);
		console.log(`max zoom is ${maxZoom}. Configure with --max-zoom`);

		if(!(imageExporterKey in imageExporters)) {
			onUsageError('Please select a tile image format.');
		}
		else {
			const imageExporter = imageExporters[imageExporterKey];

			async.eachLimit(_.range(minZoom, maxZoom), 1, (zoomLevel, cb) => {
				console.log('Starting zoom level ' + zoomLevel);
				renderzoomlevel({sceneFilePath, zoomLevel, imageExporter}, err => {
					if(err) {
						cb(err);
					}
					else {
						cb(null);
					}
				});
			}, err => {
				if(err) {
					console.error(err);
					process.exit(1);
				}
				else {
					console.log('Finished baking tile set to max zoom ' + maxZoom);
				}
			});
		}


	}
}
