#!/usr/bin/env node

// var Tiles = require('./tiles'),
const scene = require('./scene'),
      async = require('async');

var SceneImage = require('./ui/SceneImage'),
    TileImage = require('./TileImage');

const {Tile} = require('shardedmapview');

module.exports = ({sceneFilePath, zoomLevel, imageExporter}, cb) => {
	scene.open(sceneFilePath, (err, scene) => {
		if(err) {
			cb(err);
		}
		else {


			var tileSceneForTilesWithContent = {};

			async.eachLimit(scene.images, 15,
				(image, cb) => {
					
					// just ask any tile in zoom level for the scale. this
					// probably only works when using plain cartesian/euclidian coordinates.
					// won't work with spatially morphed coordinates like mercator.
					TileImage({
						tile: Tile({zoom: zoomLevel, row: 0, column: 0}),
						imageExporter
					}).sceneImageProjectionInfo(image, (err, info) => {

						if(err) {
							cb(err);
						}
						else {

							// scale <= 2 makes sure we capture all resolution
							// available into scene. at higher zoom, we get this
							// image's contents by drawing the local quadrant of
							// the parent tile first.
							//
							// also, skip if diagonal pixel size less than 1.
							// hopefully that avoids the fucking pngStream out of memory
							// issue????
							// console.log('scale: ' + info.scale);
							if(info.scale <= 2 &&
							   // info.diagonalDistanceInTilePixels > 1 &&
							   info.scale >= 0.01
							) {

								var tilesInBounds = Tile.tilesForBoundsAtZoom(SceneImage.GetBounds(image), zoomLevel);

								tilesInBounds.forEach(tile => {
									tileSceneForTilesWithContent[tile.key()] = tileSceneForTilesWithContent[tile.key()] || {
										tile: tile,
										images: []
									};
									tileSceneForTilesWithContent[tile.key()].images.push(image);
								});
							}
							cb(null);
						}

					});
				},
				err => {

					if(err) {
						cb(err);
					}
					else {
						async.eachLimit(
							Object.entries(tileSceneForTilesWithContent),
							4,
							([tileKey, tileScene], cb) => {
							    var tileImage = TileImage({
							    	tile: tileScene.tile,
							    	imageExporter
							    });
							   	
							    tileScene.images.sort(function(a, b) {
							    	var aDiag = SceneImage.DiagonalDistance(a),
							    	    bDiag = SceneImage.DiagonalDistance(b);
							    	if(aDiag.lessThan(bDiag)) {
							    		return -1;
							    	}
							    	else if(aDiag.greaterThan(bDiag)) {
							    		return 1;
							    	}
							    	else {
							    		return 0;
							    	}
							    }).reverse();

							    tileImage.drawParentQuadrant(err => {
							    	if(err) {
							    		console.error('Warning: Could not draw parent quadrant');
							    	}
									
									console.log(`drawing ${tileScene.images.length} images into ${tileKey}`)

								    async.eachSeries(tileScene.images, (image, cb) => {
								    	tileImage.drawSceneImage(image, err => {
								    		cb(err);
								    	});
								    }, err => {
								    	if(err) {
								    		console.error(err);
								    	}
								    	else {
									    	tileImage.save(err => {
									    		if(err) {
									    			cb(err);
									    		}
									    		else {
									    			cb(null);
									    		}	
									    	});
								    	}
								    	// console.log('drew '+tileScene.images.length+' images into tile ', tileScene.tile.key());
								    });
							    	
							    });


							},
							err => {
								if(err) {
									cb(err);
								}
								else {
									cb(null);
								}
							}
						);

						
					}


				}
			);

		}
	});
};

