#!/usr/bin/env node

const fs = require('fs');
const argv = require('yargs').argv;
// const mkdirp = require('mkdirp');
const path = require('path');
const async = require('async');
const sharp = require('sharp');
// const _ = require('lodash');


const sceneFilePath = path.join(process.cwd(), 'scene.json');

const Point = require('./Point');

module.exports = {
	usage: {
		commandLine: () => '',
		description: () => 'Create a scene with passed images in a row'
	},
	run: (cliArgs, {onUsageError}) => {

		require('./readdirVisible')(argv.imagesDir, (err, files) => {

			files.sort();

			if(argv.reverse) {
				files.reverse();
			}

			// console.log(files);

			let scene = [];

			async.eachSeries(files, (filename, cb) => {
				const imagePath = filename;
				sharp(imagePath)
					.metadata()
					.then(metadata => {
						const prev = scene.length ?
							scene[scene.length-1]
							: {
								lowerRight: {
									x: '0'
								}
							};
						scene.push({
							"imageUrl": imagePath,
							"imageAspect": metadata.width / metadata.height,
							"upperLeft": {
								"x": ''+(+prev.lowerRight.x),
								"y": ''+metadata.height
							},
							"lowerRight": {
								"x": ''+((+prev.lowerRight.x)+metadata.width),
								"y": ''+0
							}
						});
						cb();
					})
					.catch(err => {
						console.error('Error while trying to read image ' + imagePath)
					});
			}, err => {
				if(err) {
					console.error(err);
					process.exit(1);
				}
				else {
					const sceneJson = JSON.stringify(scene, null, 2);
					fs.writeFile('scene.json', sceneJson, {encoding: 'utf8'}, err => {
						if(err) {
							console.error(err);
							process.exit(1);
						}
						else {
							console.error('Saved scene.json');
						}
					});
				}
			});
		});
	}
}
