(function (name, definition){
  if (typeof define === 'function'){ // AMD
    define(definition);
  } else if (typeof module !== 'undefined' && module.exports) { // Node.js
    module.exports = definition();
  } else { // Browser
    var theModule = definition(), global = this, old = global[name];
    theModule.noConflict = function () {
      global[name] = old;
      return theModule;
    };
    global[name] = theModule;
  }
})('SceneImage', function () {

  // breaks UI. let's make the UI webpacked finally.
  const Decimal = require('shardedmapview').DecimalConfigured;

  const D = num => new Decimal(num);

  const PI = Decimal.acos(-1);
  
  function distance(a, b) {
    return Decimal.hypot(D(a.y).minus(b.y), D(a.x).minus(b.x));
  }

  var api = {
    
    GetBounds: function(sceneImage) {
      var corners = [
        sceneImage.upperLeft,
        sceneImage.lowerRight,
        api.CalculateThirdCorner(sceneImage.upperLeft, sceneImage.lowerRight, sceneImage.imageAspect),
        api.CalculateThirdCorner(sceneImage.lowerRight, sceneImage.upperLeft, sceneImage.imageAspect)
      ];
      var xValues = corners.map(function(coord) {
        return coord.x;
      });
      var yValues = corners.map(function(coord) {
        return coord.y;
      });
      return {
        left: Decimal.min.apply(Decimal, xValues),
        right: Decimal.max.apply(Decimal, xValues),
        top: Decimal.max.apply(Decimal, yValues),
        bottom: Decimal.min.apply(Decimal, yValues)
      };
    },

    DiagonalDistance: function(sceneImage) {
      return distance(sceneImage.upperLeft, sceneImage.lowerRight);
    },

    CompareDiagonals: function(a, b) {
      const aDist = api.DiagonalDistance(a),
            bDist = api.DiagonalDistance(b);
      if(aDist.lessThan(bDist)) {
        return -1;
      }
      else if(aDist.greaterThan(bDist)) {
        return 1;
      }
      else {
        return 0;
      }
    },

    GetRotation: function(sceneImage) {
      const l = D(sceneImage.upperLeft.y).minus(sceneImage.lowerRight.y),
            r = D(sceneImage.lowerRight.x).minus(sceneImage.upperLeft.x);
      const p = Decimal.atan2(l, r);
      const n = Decimal.atan(sceneImage.imageAspect);
      return PI.dividedBy(2).minus(n.plus(p)).toNumber();
    },

    GetUpperLeft: function(sceneImage) {
      return sceneImage.upperLeft;
    },

    CalculateThirdCorner: function(b, a, aspect) {
      const l = D(a.y).minus(b.y),
            r = D(b.x).minus(a.x);
      const p = Decimal.atan2(l, r);
      const n = Decimal.atan(aspect),
            m = PI.dividedBy(2).minus(n.plus(p));
      const s = distance(a, b).times(Decimal.cos(n));
      const z = s.times(Decimal.sin(m)),
            q = s.times(Decimal.cos(m));
      return {
        y: D(b.y).plus(q),
        x: D(b.x).minus(z)
      };
    }

  };

  return api;

});