const Decimal = require('shardedmapview').DecimalConfigured;
const path = require('path');

const {Shards, Bounds} = require('shardedmapview');
const SceneImage = require('./SceneImage');

const D = num => new Decimal(num);

var mymap = L.map('placer-map', {
	zoominfoControl: true,
	zoomControl: false,
    crs: L.CRS.Simple,
    maxZoom: 50,
    zoomAnimation: false
});

L.control.coordinates({
	position:"bottomleft", //optional default "bootomright"
	decimals:2, //optional default 4
	decimalSeperator:".", //optional default "."
	labelTemplateLat:"Y: {y}", //optional default "Lat: {y}"
	labelTemplateLng:"X: {x}", //optional default "Lng: {x}"
	enableUserInput:false, //optional default true
	useDMS:false, //optional default false
	useLatLngOrder: true, //ordering of labels, default false-> lng-lat
	markerType: L.marker, //optional default L.marker
	markerProps: {}, //optional default {},
}).addTo(mymap);

let currentShard;

let selectedImage;

function onShardZoomEnd() {
	const zoom = mymap.getZoom();
	updateViewerShardAndUrl();
	updateCulling();
}

let checkerboardLayer;

const initializeShard = ({shard, globalView}) => {
	const newShardView = shard.globalViewToShardView(globalView);
	mymap.setView(xYToLatLng(newShardView.center), newShardView.zoom);
	currentShard = shard;
	
	checkerboardLayer = new L.TileLayer.Functional(function (view) {

		var parity = Math.abs(view.tile.row + view.tile.column) % 2;
		var backgroundColor = ['ffffff', 'cccccc'][parity],
		    foregroundColor = ['888888', '888888'][parity];
		const globalZoom = `glob. zoom: ${Shards.ShardZoomToGlobalZoom(view.zoom, shard)}`;
		var text = globalZoom + ', local r/c: ' + view.tile.row + ', ' + view.tile.column;
		return 'https://placeholdit.co/i/256x256?&bg='+backgroundColor+'&fc='+foregroundColor+'&text='+text;
	  
	}, {maxZoom: 50}).addTo(mymap);

	mymap.on('zoomend', onShardZoomEnd);
}

const deInitializeShard = (shard) => {
	mymap.off('zoomend', onShardZoomEnd);
	checkerboardLayer.removeFrom(mymap);
};

const updateUrl = ({globalCenter, globalZoom}) => {
	const hashIndex = window.location.href.indexOf('#');
	const viewString = `${globalZoom},${globalCenter.x.toString()},${globalCenter.y.toString()}`;
	const base = hashIndex > -1 ?
		window.location.href.substring(0, hashIndex) :
		window.location.href;
	history.replaceState(null, '', `${base}#${viewString}`);
};

const updateViewerShardAndUrl = () => {
	const globalCenter = Shards.ShardCoordToGlobalCoord(latLngToXY(mymap.getCenter()), currentShard);
	const globalZoom = Shards.ShardZoomToGlobalZoom(mymap.getZoom(), currentShard);
	const newShard = Shards.ShardForGlobalView({center: globalCenter, zoom: globalZoom});
	if(!newShard.isEqual(currentShard)) {
		console.log('new shard is different, initilializing new shard');
		deInitializeShard(currentShard);
		initializeShard({
			shard: newShard,
			globalView: {
				center: globalCenter,
				zoom: globalZoom
			}
		});
	}
	updateUrl({globalCenter, globalZoom});
};

const hashIndex = window.location.href.indexOf('#');
let initialGlobalView;
if(hashIndex > -1) {
	const parts = window.location.href.substring(hashIndex+1).split(',');
	const initialZoom = parts[0];
	const initialX = D(parts[1]);
	const initialY = D(parts[2]);
	initialGlobalView = {
		center: {
			x: initialX,
			y: initialY
		},
		zoom: initialZoom
	};
} 
else {
	initialGlobalView = {
		center: {
			x: 128,
			y: -128
		},
		zoom: 10
	};
}
initializeShard({
	shard: Shards.ShardForGlobalView(initialGlobalView),
	globalView: initialGlobalView
});




var images = [];

function latLngToXY(latLng) {
	return {
		x: latLng.lng,
		y: latLng.lat
	};
}
function xYToLatLng(xy) {
	return {
		lng: xy.x,
		lat: xy.y
	};
}

function CalculateThirdCorner(a, b, aspect) {
	return SceneImage.CalculateThirdCorner(
		a,
		b,
		aspect
	);
}

function placeImage(imageUrl, imageAspect, upperLeft, lowerRight) {

	var upperLeftMarker, lowerRightMarker, overlay;

	let activeShard;

	const overlayCoords = () => [
		Shards.GlobalCoordToShardCoord(
			upperLeft,
			activeShard
		),
		Shards.GlobalCoordToShardCoord(
			CalculateThirdCorner(
				lowerRight,
				upperLeft,
				imageAspect
			),
			activeShard
		),
		Shards.GlobalCoordToShardCoord(
			CalculateThirdCorner(
				upperLeft,
				lowerRight,
				imageAspect
			),
			activeShard
		)
	].map(xYToLatLng);

	function repositionImage() {

		function repositionModelAndOverlay(_upperLeft, _lowerRight) {
			upperLeft = _upperLeft;
			lowerRight = _lowerRight;
			overlay.reposition(
				overlayCoords()[0],
				overlayCoords()[1],
				overlayCoords()[2]
			);
		}

		repositionModelAndOverlay(
			Shards.ShardCoordToGlobalCoord(latLngToXY(upperLeftMarker.getLatLng()), activeShard),
			Shards.ShardCoordToGlobalCoord(latLngToXY(lowerRightMarker.getLatLng()), activeShard)
		);
		save();
	};

	var placedImage = {

		showInShard(shard) {
			activeShard = shard;
			upperLeftMarker = L.marker(
				xYToLatLng(
					Shards.GlobalCoordToShardCoord(
						upperLeft,
						shard
					)
				),
				{draggable: true}
			);
			lowerRightMarker = L.marker(
				xYToLatLng(
					Shards.GlobalCoordToShardCoord(
						lowerRight,
						shard
					)
				),
				{draggable: true}
			);
			overlay = L.imageOverlay.rotated(
				'file/' + imageUrl,
				overlayCoords()[0],
				overlayCoords()[1],
				overlayCoords()[2],
				{
					opacity: 0.7,
					interactive: true
				}
			);
			upperLeftMarker.addTo(mymap);
			lowerRightMarker.addTo(mymap);
			overlay.addTo(mymap);

			const markerDragged = () => {
				repositionImage();
				selectedImage = placedImage;
			};

			upperLeftMarker.on('drag dragend', markerDragged);
			lowerRightMarker.on('drag dragend', markerDragged);

			// these should be factored down to just updateDisplayZ i think
			upperLeftMarker.on('dragend', updateCulling);
			lowerRightMarker.on('dragend', updateCulling);

		},

		hide() {
			if(overlay) {
				overlay.removeFrom(mymap);
				overlay = null;
			}
			if(upperLeftMarker) {
				upperLeftMarker.removeFrom(mymap);
				upperLeftMarker.off('drag dragend', repositionImage);
				upperLeftMarker.off('dragend', updateCulling);
				upperLeftMarker = null;
			}
			if(lowerRightMarker) {
				lowerRightMarker.removeFrom(mymap);
				lowerRightMarker.off('drag dragend', repositionImage);
				lowerRightMarker.off('dragend', updateCulling);
				lowerRightMarker = null;
			}
			activeShard = null;
		},

		getDiagonalDistance() {
			return distance(upperLeft, lowerRight);
		},

		serialize: function() {
			return {
				imageUrl: path.join('placed', imageUrl),
				imageAspect: imageAspect,
				upperLeft: {
					x: upperLeft.x.toString(),
					y: upperLeft.y.toString()
				},
				lowerRight: {
					x: lowerRight.x.toString(),
					y: lowerRight.y.toString()
				}
			};
		}
	};

	images.push(placedImage);

	save();

	return placedImage;
}

window.addEventListener('keyup', e => {
	if(selectedImage && e.key === 'Backspace') {
		console.info("let's delete ", selectedImage);
		selectedImage.hide();
		images.splice(images.indexOf(selectedImage), 1);
		save();
	}
});

function BoundsFromLeafletBounds(leafletBounds) {
	return Bounds({
		left: leafletBounds.getWest(),
		right: leafletBounds.getEast(),
		top: leafletBounds.getNorth(),
		bottom: leafletBounds.getSouth()
	});
}

function placeImageInCurrentBounds(url, aspect) {
	var viewBounds = GetGlobalViewportBounds();
	var placedBounds = viewBounds.containedBoundsWithAspect(aspect);
	placeImage(url, aspect, placedBounds.getUpperLeft(), placedBounds.getLowerRight()).showInShard(currentShard);
}



var myDropzone = new Dropzone("div.file-import-zone", { 
    url: '/file',
    autoProcessQueue: false
});
myDropzone.on("addedfile", function(file) {
	var formData = new FormData();
	formData.append(file.name, file);
	superagent.post('/file/' + file.name)
		.send(formData)
		.end(function(err, response) {
			var img = new Image();
			img.onload = function() {
				console.log(img.width, img.height);
				placeImageInCurrentBounds(file.name, img.width / img.height);
				myDropzone.removeAllFiles();
			}
			img.src = 'file/' + file.name;
		});
});

const CompareDiagonals = (a, b) => {
	if(a.getDiagonalDistance().lessThan(b.getDiagonalDistance())) {
		return -1;
	}
	if(a.getDiagonalDistance().greaterThan(b.getDiagonalDistance())) {
		return 1;
	}
	return 0;
};

const GetGlobalViewportBounds = () => {
	// const boundsTransform = Shards.ShardLocalBounds.boundsTransformerTo(currentShard.globalBounds());
	// const globalViewportBounds = boundsTransform(BoundsFromLeafletBounds(mymap.getBounds()));
	const shardViewportBounds = BoundsFromLeafletBounds(mymap.getBounds());
	const globalUpperLeft = currentShard.localCoordToGlobalCoord(shardViewportBounds.getUpperLeft());
	const globalLowerRight = currentShard.localCoordToGlobalCoord(shardViewportBounds.getLowerRight());
	const globalViewportBounds = Bounds({
		left: globalUpperLeft.x,
		right: globalLowerRight.x,
		top: globalUpperLeft.y,
		bottom: globalLowerRight.y
	});
	return globalViewportBounds;
};

function updateCulling() {
	const globalViewportBounds = GetGlobalViewportBounds();
	const viewportDiagonal = globalViewportBounds.diagonal();

	const low = D(0.02).times(viewportDiagonal);
	const high = D(20).times(viewportDiagonal);
	images.sort(CompareDiagonals).reverse();
	images.forEach(placedImage => placedImage.hide());
	let showCount = 0;
	images.forEach(placedImage => {
		const placedImageDiagonal = placedImage.getDiagonalDistance();

		if(low.lessThan(placedImageDiagonal) && placedImageDiagonal.lessThan(high)) {
			placedImage.showInShard(currentShard);
			showCount++;
		}
	});
}

function sortSerializedSceneByDiagonalLength(serializedScene) {
	return serializedScene.sort(SceneImage.CompareDiagonals);
}

function deserializeScene(serializedScene) {
	sortSerializedSceneByDiagonalLength(serializedScene).forEach(placedImage => {
		var img = placeImage(
			placedImage.imageUrl,
			placedImage.imageAspect,
			placedImage.upperLeft,
			placedImage.lowerRight
		);
	});
	updateCulling();
}

function serializeScene() {
	return sortSerializedSceneByDiagonalLength(
		images.map(placedImage => placedImage.serialize())
	);
}

var sceneSlug;

function save() {
	superagent.post('/scene')
		.set('Content-type', 'application/json')
		.send(serializeScene())
		.end(function(err, response) {
			//upload done!
		});
}

superagent.get('/scene').end(function(err, response) {
	if(err) {
		throw new Error('Could not get scene from /scene');
	}
	else {
		deserializeScene(response.body);
	}
});

function distance(a, b) {
	return Decimal.hypot(D(a.x).minus(b.x), D(a.y).minus(b.y));
}
